﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'  CsvRecordクラス
'
'  Copyright (c) 2016, Advanced Core Technologies, Inc. All Rights Reserved.
'
'  2016/02/04 Rev.1.0.0.0 MOTO VB.NET版作成
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

Imports System.Text

Namespace Act.VisualBasic.Utilities.CsvIO

    ''' <summary>
    ''' CSVファイルのレコードを表します。
    ''' </summary>
    Public Class CsvRecord

        '==========================================================================================
        '  Constructors
        '==========================================================================================
#Region "Constructors"

        Public Sub New(Optional ByVal delimiter As Delimiter = Delimiter.Comma,
                       Optional ByVal quote As QuotationMark = CsvIO.QuotationMark.Double,
                       Optional ByVal tokens As IEnumerable(Of String) = Nothing)
            Me.Delimiter = delimiter
            Me.QuotationMark = quote

            If tokens Is Nothing Then
                Return
            End If
            For Each token In tokens
                Add(token)
            Next

        End Sub

#End Region

        '==========================================================================================
        '  Private Constants
        '==========================================================================================
#Region "Private Constants"

        Private Shared ReadOnly DoubleQuote As Char = """"c

#End Region

        '==========================================================================================
        '  Private Fields
        '==========================================================================================
#Region "Private Fields"

        ''' <summary>
        ''' トークンのバッキングフィールドです。
        ''' </summary>
        Private tokens_ As New List(Of String)

#End Region

        '==========================================================================================
        '  Properties
        '==========================================================================================
#Region "Properties"

        ''' <summary>
        ''' トークンの数を取得します。
        ''' </summary>
        Public ReadOnly Property Count As Integer
            Get
                Return tokens_.Count
            End Get
        End Property

        ''' <summary>
        ''' 区切り文字の種別を取得します。
        ''' </summary>
        Public Property Delimiter As Delimiter

        ''' <summary>
        ''' 区切り文字を取得します。
        ''' </summary>
        Public ReadOnly Property DelimiterCharacter As Char
            Get
                Dim delimiterChar As Char = vbNullChar
                Select Case Delimiter
                    Case CsvIO.Delimiter.Comma
                        delimiterChar = ","c
                    Case CsvIO.Delimiter.Tab
                        delimiterChar = vbTab
                End Select

                Return delimiterChar
            End Get
        End Property

        ''' <summary>
        ''' 引用符を取得または設定します。
        ''' </summary>
        Public Property QuotationMark As QuotationMark

        ''' <summary>
        ''' 指定したインデックスのトークンを取得または設定します。
        ''' </summary>
        ''' <param name="index">インデックス番号</param>
        ''' <value>トークンに設定する文字列</value>
        ''' <returns>指定したインデックス位置にあるトークン。範囲外を指定した場合はnull。</returns>
        Default Public Property Item(ByVal index As Integer) As String
            Get
                Dim token As String = Nothing
                If index >= 0 AndAlso index < Count Then
                    token = tokens_(index)
                End If

                Return token
            End Get
            Set(ByVal value As String)
                If index >= 0 AndAlso index < Count Then
                    tokens_(index) = value
                End If
            End Set
        End Property

#End Region

        '==========================================================================================
        '  Public Methods
        '==========================================================================================
#Region "Public Methods"

        ''' <summary>
        ''' レコードの末尾にvalueを追加します。
        ''' </summary>
        ''' <param name="value">追加するトークン</param>
        Public Sub Add(ByVal value As String)
            If value Is Nothing Then
                value = String.Empty
            End If
            tokens_.Add(value)
        End Sub

        ''' <summary>
        ''' index番目のトークンを削除します。
        ''' </summary>
        ''' <param name="index">削除するインデックス番号</param>
        Public Sub RemoveAt(ByVal index As Integer)
            If index < 0 OrElse index >= Count Then
                Throw New ArgumentOutOfRangeException("index")
            End If
            tokens_.RemoveAt(index)
        End Sub

        ''' <summary>
        ''' 全てのトークンを削除します。
        ''' </summary>
        Public Sub Clear()
            tokens_.Clear()
        End Sub

        ''' <summary>
        ''' index番目にvalueのトークンを挿入します。
        ''' </summary>
        ''' <param name="index">挿入するインデックス位置</param>
        ''' <param name="value">追加するトークン</param>
        Public Sub Insert(ByVal index As Integer, ByVal value As String)
            If index < 0 OrElse index >= Count Then
                Throw New ArgumentOutOfRangeException("index")
            End If
            If value Is Nothing Then
                value = String.Empty
            End If

            tokens_.Insert(index, value)
        End Sub

        ''' <summary>
        ''' インスタンスの値を文字列で返します。
        ''' </summary>
        ''' <returns>区切り文字で区切られた文字列。トークンがない場合は空文字。</returns>
        Public Overrides Function ToString() As String
            If Count = 0 Then
                Return String.Empty
            End If

            Dim builder As New StringBuilder()
            For Each token In tokens_
                ' 二重引用符をエスケープし、引用符でトークンを囲い、末尾に区切り文字を追加
                Dim escaped As String = AddEscapeDoubleQuote(token)
                builder.Append(EnclosedWithinQuotes(escaped, QuotationMark))
                builder.Append(DelimiterCharacter)
            Next
            ' 最後の区切り文字を削除
            builder.Remove(builder.Length - 1, 1)

            Return builder.ToString()
        End Function

        ''' <summary>
        ''' 全てのトークンを返します。
        ''' </summary>
        ''' <returns>トークンを格納した配列。トークンがない場合はnull。</returns>
        Public Function GetAllTokens() As String()
            If Count = 0 Then
                Return Nothing
            End If

            Return tokens_.ToArray()
        End Function

#End Region

        '==========================================================================================
        '  Private Methods
        '==========================================================================================
#Region "Private Methods"

        ''' <summary>
        ''' 引用符で囲んだ文字列を返します。
        ''' </summary>
        ''' <param name="value">対象の文字列</param>
        ''' <param name="quotationMark">引用符</param>
        ''' <returns>引用符で囲んだ文字列。</returns>
        Private Shared Function EnclosedWithinQuotes(ByVal value As String,
                                                     ByVal quotationMark As QuotationMark) As String
            If value Is Nothing Then
                Throw New ArgumentNullException("value")
            End If
            If quotationMark = CsvIO.QuotationMark.None Then
                Return value
            End If

            Return DoubleQuote & value & DoubleQuote
        End Function

        ''' <summary>
        ''' 二重引用符をエスケープした文字列を返します。
        ''' </summary>
        ''' <param name="value">対象の文字列</param>
        ''' <returns>二重引用符をエスケープした文字列。</returns>
        Private Shared Function AddEscapeDoubleQuote(ByVal value As String) As String
            If value Is Nothing Then
                Throw New ArgumentNullException("value")
            End If

            Return value.Replace("""", """""")
        End Function

#End Region

    End Class

    '==========================================================================================
    '  Enum
    '==========================================================================================
#Region "Enum"

    ''' <summary>
    ''' 区切り文字を指定します。
    ''' </summary>
    Public Enum Delimiter
        ''' <summary>
        ''' カンマで区切ります。
        ''' </summary>
        Comma

        ''' <summary>
        ''' タブ文字で区切ります。
        ''' </summary>
        Tab
    End Enum

    ''' <summary>
    ''' 引用符の文字を指定します。
    ''' </summary>
    Public Enum QuotationMark

        ''' <summary>
        ''' 二重引用符を指定します。
        ''' </summary>
        ''' <remarks></remarks>
        [Double]

        ''' <summary>
        ''' 引用符なしを指定します。
        ''' </summary>
        ''' <remarks></remarks>
        None

    End Enum

#End Region

End Namespace

