﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'  ACT距離計算サービス サンプルプログラム
'  MainFormクラス
'
'  Copyrights (c) 2016-2023, Advanced Core Technologies, Inc. All rights reserved.
'  
'  2023/09/21 Rev.1.0.0.1 KYON AdminURLに合わせてSSL使用フラグを変更出来るように変更
'  2016/02/12 Rev.1.0.0.0 MOTO 作成
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

Imports System.IO
Imports System.Text
Imports ACTSOAPVBSearchAddress.Act.VisualBasic.Utilities.CsvIO
Imports ACTSOAPVBSearchAddress.ArrayExtensionMethods

Public Class MainForm

    '==============================================================================================
    '  Private Fields
    '==============================================================================================
#Region "Private Fields"

    ''' <summary>
    ''' ユーザー情報です。
    ''' </summary>
    Private userSettings_ As UserSettings

    ''' <summary>
    ''' 距離計算サービスのユーザーです。
    ''' </summary>
    Private user_ As User

#End Region

    '==============================================================================================
    '  Private Constants
    '==============================================================================================
#Region "Private Constants"

    ''' <summary>
    ''' 地点名の列番号です。
    ''' </summary>
    Private Shared ReadOnly NameColumnIndex As Integer = 0

    ''' <summary>
    ''' 住所の列番号です。
    ''' </summary>
    Private Shared ReadOnly AddressColumnIndex As Integer = 1

#End Region

    '==============================================================================================
    '  Private Methods
    '==============================================================================================
#Region "Private Methods"

    ''' <summary>
    ''' 検索を実行し実行結果をファイルに出力します。
    ''' </summary>
    ''' <param name="userSettings">ユーザー設定</param>
    ''' <returns>実行結果のメッセージ。</returns>
    Private Function DoSearch(ByVal userSettings As UserSettings) As String
        Dim message As New StringBuilder()
        Try
            '--------------------------------------------------------------------------------------
            '  管理サービスオブジェクトと環境設定オブジェクトの作成
            '--------------------------------------------------------------------------------------
            ' 管理サービスオブジェクトを作成
            Dim adminService As New DCAdmin.ACTDCADMIN()
            ' エンドポイントURLを設定
            adminService.Url = user_.AdminUrl

            ' 環境設定オブジェクトを作成
            Dim adminEnviron As DCAdmin.Environ = user_.GetAdminEnviron()

            '--------------------------------------------------------------------------------------
            '  ユーザー情報オブジェクトの取得
            '--------------------------------------------------------------------------------------
            ' ユーザー情報オブジェクトを作成
            Dim userInfo As New DCAdmin.UserInformation2()
            ' ユーザー情報取得サービスの呼び出し
            If Not adminService.GetUserInformation2(adminEnviron, userInfo) Then
                ' ユーザー情報の取得に失敗した場合
                message.AppendLine("ユーザー情報の取得に失敗しました。")
                Return message.ToString()
            End If
            user_.UserInfo = userInfo

            '--------------------------------------------------------------------------------------
            '  住所検索サービスオブジェクトと住所検索サービス環境設定構造体オブジェクトの作成
            '--------------------------------------------------------------------------------------
            ' 住所検索サーバー取得サービスの呼び出し
            If Not adminService.GetUsableGeoServer(adminEnviron) Then
                ' 住所検索サーバーの取得に失敗した場合
                message.AppendLine("住所検索サーバーの情報取得に失敗しました。")
                Return message.ToString()
            End If

            ' 距離計算サービスオブジェクトの作成
            Dim gcService As New GCService.ACTGCWS()
            ' エンドポイントURLを設定
            gcService.Url = Utilities.ChangeProtocol(adminEnviron.WSURL, CalcSettings.UseSsl)
            message.AppendLine("住所検索サーバー: " & gcService.Url & "(" & adminEnviron.Host &
                               ":" & adminEnviron.Port & ")")

            ' 住所検索サービス環境設定構造体オブジェクトを作成
            Dim gcEnviron As GCService.Environ = Utilities.GetGCServiceEnviron(adminEnviron)

            '--------------------------------------------------------------------------------------
            '  CSVファイルを読み込み住所検索アイテム構造体配列を作成
            '--------------------------------------------------------------------------------------
            ' ファイルから住所を読み込む
            Dim records() As CsvRecord = ReadCsvFile(userSettings.InputFilePath)
            If records Is Nothing Then
                ' CSVファイルの読み込みに失敗した場合
                message.AppendLine("入力ファイルの読み込みに失敗しました。")
                Return message.ToString()
            End If

            ' 住所検索アイテム構造体に読み取った経度緯度情報を設定
            Dim items() As GCService.SearchAddressItem = CreateItems(records)

            '--------------------------------------------------------------------------------------
            '  検索住所レベルの設定
            '--------------------------------------------------------------------------------------
            ' ユーザーの最大検索住所レベルを検索住所レベルに設定
            Dim searchAddressLevel As Integer = user_.UserInfo.GeoMaxAddressLevel
            message.AppendLine("検索住所レベル: " & searchAddressLevel)

            '--------------------------------------------------------------------------------------
            '  共通パラメータ構造体の設定
            '--------------------------------------------------------------------------------------
            ' 共通パラメータ構造体オブジェクトを作成
            Dim param As New GCService.CommonParam With {.Datum = CalcSettings.Datum,
                                                         .CoordinateFormat =
                                                            CalcSettings.CoordinateFormat,
                                                         .SearchFlag = CalcSettings.SearchFlag}


            '--------------------------------------------------------------------------------------
            '  住所検索の実行＆実行結果の取得
            '--------------------------------------------------------------------------------------
            ' 検索結果を格納する変数を作成
            Dim geoInfo As GCService.GeoInfo = Nothing

            ' 計算可能地点数を取得
            Dim maxLocs As Integer = user_.UserInfo.GeoMaxLocs

            ' 必要計算回数を求める
            Dim numSearch As Integer = Math.Ceiling(CDbl(items.Length) / CDbl(maxLocs))

            ' 分割して検索を実行
            For i As Integer = 0 To numSearch - 1
                ' 対象地点を設定
                Dim start As Integer = maxLocs * i
                Dim length As Integer = If(i + 1 = numSearch, items.Length - start, maxLocs)
                Dim target() As GCService.SearchAddressItem = items.Copy(start, length)

                ' 住所検索を実行
                Dim result As Integer = gcService.SearchAddress(gcEnviron, param,
                                                                searchAddressLevel, target, geoInfo)

                ' 実行結果の取得
                If result = 0 Then
                    ' 成功
                    message.AppendLine(String.Format("{0} / {1} 回目：成功", i + 1, numSearch))
                Else
                    ' 失敗、エラーメッセージを取得
                    message.AppendLine(String.Format("{0} / {1} 回目：失敗{2}{3}", i + 1, numSearch,
                                                     Environment.NewLine,
                                                     gcService.GetGCWSMessage(gcEnviron, result)))
                End If

                ' 実行結果を元の配列に書き戻す
                Array.Copy(target, 0, items, start, length)

                ' 最後のループ以外は待機
                If i + 1 <> numSearch Then
                    ' ミリ秒単位の待機時間を取得
                    Dim waitTime As Integer = user_.UserInfo.GeoInterval * 1000
                    ' 待機
                    System.Threading.Thread.Sleep(waitTime)
                End If
            Next

            ' 最終住所検索日時を更新
            user_.UpdateLastGCTime()

            '--------------------------------------------------------------------------------------
            '  ファイル出力
            '--------------------------------------------------------------------------------------
            If Not WriteItems(userSettings.OutputFilePath, items) Then
                ' 出力に失敗した場合
                message.AppendLine("ファイルの書き込みに失敗しました。")
            End If

        Catch ex As System.Net.WebException
            message.AppendLine("通信エラーが発生しました。")
            message.AppendLine(ex.Message)
        Catch
            '
        End Try

        Return message.ToString()
    End Function

    ''' <summary>
    ''' 検索を実行し実行結果をファイルに出力します。
    ''' </summary>
    ''' <param name="userSettings">ユーザー設定</param>
    ''' <returns>実行結果のメッセージ。</returns>
    Private Function DoWork(ByVal userSettings As UserSettings) As String
        Dim message As String = String.Empty
        ' 進捗フォームを表示して計算を実行
        ProgressDialog.Run(Sub()
                               message = DoSearch(userSettings)
                           End Sub, "検索中")

        Return message
    End Function

    ''' <summary>
    ''' CSVファイルの全レコードを読み取り結果を返します。
    ''' </summary>
    ''' <param name="path">対象ファイルのパス</param>
    ''' <returns>読み取り結果のCsvRecord配列。読み取りに失敗した場合はnull。</returns>
    Private Shared Function ReadCsvFile(ByVal path As String) As CsvRecord()
        Dim records() As CsvRecord = Nothing
        Try
            Using strmReader As New StreamReader(path, Encoding.GetEncoding("shift_jis"))
                Using reader As New CsvReader(strmReader, False, Delimiter.Comma,
                                              QuotationMark.Double)
                    records = reader.ReadAllRecords()
                End Using
            End Using
        Catch
            '
        End Try

        Return records
    End Function

    ''' <summary>
    ''' CsvRecord配列から住所検索アイテム構造体配列を作成し返します。
    ''' </summary>
    ''' <param name="records">CsvRecord配列</param>
    ''' <returns>住所検索アイテム構造体配列。</returns>
    Private Shared Function CreateItems(ByVal records As CsvRecord()
                                        ) As GCService.SearchAddressItem()
        ' 住所検索アイテム構造体に住所と地点名を設定
        Dim items(records.Length - 1) As GCService.SearchAddressItem
        For i As Integer = 0 To items.Length - 1
            items(i) = New GCService.SearchAddressItem
            items(i).SearchAddress = records(i)(AddressColumnIndex)
            items(i).Tag = records(i)(NameColumnIndex)
        Next

        Return items
    End Function

    ''' <summary>
    ''' 住所検索アイテム構造体をCSVファイルに書き込みます。
    ''' </summary>
    ''' <param name="path">出力ファイルのパス</param>
    ''' <param name="items">計算結果を格納した住所検索アイテム構造体配列</param>
    ''' <returns>書き込みに成功した場合はtrue、失敗した場合はfalse。</returns>
    Private Shared Function WriteItems(ByVal path As String,
                                       ByVal items As IEnumerable(Of GCService.SearchAddressItem)
                                       ) As Boolean
        Try
            Using strmWriter As New StreamWriter(path, False, Encoding.GetEncoding("shift_jis"))
                Using writer As New CsvWriter(strmWriter)
                    For Each item In items
                        Dim record As New CsvRecord(Delimiter.Comma, QuotationMark.Double)
                        record.Add(item.Tag)
                        record.Add(item.SearchAddress)
                        record.Add(item.Lon)
                        record.Add(item.Lat)

                        writer.WriteRecord(record)
                    Next
                End Using
            End Using
        Catch
            Return False
        End Try

        Return True
    End Function

    ''' <summary>
    ''' コントロールからユーザー設定を取得します。
    ''' </summary>
    ''' <returns>取得したUserSettingsオブジェクト。</returns>
    Private Function GetUserSettingsFromControls() As UserSettings
        Return New UserSettings() With {.UserID = textBoxUserId.Text,
                                        .Password = textBoxPassword.Text,
                                        .AdminUrl = textBoxAdminServerUrl.Text,
                                        .InputFilePath = textBoxInputFilePath.Text,
                                        .OutputFilePath = textBoxOutputFilePath.Text}
    End Function

    ''' <summary>
    ''' 指定したユーザー設定をコントロールに適用します。
    ''' </summary>
    ''' <param name="settings">適用するUserSettingsオブジェクト</param>
    Private Sub ApplyUserSettingsToControls(ByVal settings As UserSettings)
        textBoxUserId.Text = settings.UserID
        textBoxAdminServerUrl.Text = settings.AdminUrl
        textBoxInputFilePath.Text = settings.InputFilePath
        textBoxOutputFilePath.Text = settings.OutputFilePath
    End Sub

    ''' <summary>
    ''' 入力ファイルを選択し、選択したファイルのパスを返します。
    ''' </summary>
    ''' <param name="oldPath">現在の入力ファイルパス</param>
    ''' <returns>選択したファイルのパス。キャンセルの場合はoldPath。</returns>
    Private Shared Function SelectInputFile(ByVal oldPath As String) As String
        Dim newPath As String = oldPath
        Using dialog As New OpenFileDialog()
            If File.Exists(oldPath) Then
                dialog.InitialDirectory = Path.GetDirectoryName(oldPath)
                dialog.FileName = Path.GetFileName(oldPath)
            End If
            dialog.Title = "入力ファイルの選択"
            dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*"

            If (dialog.ShowDialog() = DialogResult.OK) Then
                newPath = dialog.FileName
            End If
        End Using

        Return newPath
    End Function

    ''' <summary>
    ''' 出力ファイルを選択し、選択したファイルのパスを返します。
    ''' </summary>
    ''' <param name="oldPath">現在の出力ファイルパス</param>
    ''' <returns>選択したファイルのパス。キャンセルの場合はoldPath。</returns>
    Private Shared Function SelectOutputFile(ByVal oldPath As String) As String
        Dim newPath As String = oldPath
        Using dialog As New SaveFileDialog()
            If File.Exists(oldPath) Then
                dialog.InitialDirectory = Path.GetDirectoryName(oldPath)
                dialog.FileName = Path.GetFileName(oldPath)
            End If
            dialog.Title = "出力ファイルの選択"
            dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*"

            If (dialog.ShowDialog() = DialogResult.OK) Then
                newPath = dialog.FileName
            End If
        End Using

        Return newPath
    End Function

    ''' <summary>
    ''' ユーザー設定を検証します。
    ''' </summary>
    ''' <param name="userID">ユーザーID</param>
    ''' <param name="password">パスワード</param>
    ''' <param name="adminUrl">管理サーバーURL</param>
    ''' <param name="inputFilePath">入力ファイルのパス</param>
    ''' <param name="outputFilePath">出力ファイルのパス</param>
    ''' <returns>設定が全て有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateSettings(ByVal userID As String, ByVal password As String,
                                             ByVal adminUrl As String,
                                             ByVal inputFilePath As String,
                                             ByVal outputFilePath As String) As String
        Dim message As New StringBuilder()
        If String.IsNullOrEmpty(userID) Then
            message.AppendLine("ユーザーIDが指定されていません。")
        End If
        If String.IsNullOrEmpty(password) Then
            message.AppendLine("パスワードが指定されていません。")
        End If
        If String.IsNullOrEmpty(adminUrl) Then
            message.AppendLine("管理サーバーURLが指定されていません。")
        End If
        Dim inputFilePathError = ValidateInputFilePath(inputFilePath)
        If inputFilePathError IsNot Nothing Then
            message.AppendLine(inputFilePathError)
        End If
        Dim outputFilePathError = ValidateOutputFilePath(outputFilePath)
        If outputFilePathError IsNot Nothing Then
            message.AppendLine(outputFilePathError)
        End If
        If message.Length = 0 Then
            If ConfirmOverwrite(outputFilePath) Then
                Return Nothing
            End If
            message.AppendLine("指定された " & outputFilePath & " は既に存在します。")
        End If

        Return message.ToString()
    End Function

    ''' <summary>
    ''' 入力ファイルのパスを検証します。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateInputFilePath(ByVal value As String) As String
        If String.IsNullOrEmpty(value) Then
            Return "入力ファイルが指定されていません。"
        End If
        Try
            If Not File.Exists(value) Then
                Return "指定された " & value & " が存在しません。"
            End If
        Catch
            Return "入力ファイルのパスが不正です。"
        End Try

        Return Nothing
    End Function

    ''' <summary>
    ''' 出力ファイルのパスを検証します。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateOutputFilePath(ByVal value As String) As String
        If String.IsNullOrEmpty(value) Then
            Return "出力ファイルが指定されていません。"
        End If
        Try
            Dim directoryName As String = Path.GetDirectoryName(value)
            If String.IsNullOrEmpty(directoryName) Then
                Return "指定されたフォルダは存在しません。"
            End If
            If Not Directory.Exists(directoryName) Then
                Return "指定された " & directoryName & " が存在しません。"
            End If
        Catch
            Return "出力ファイルのパスが不正です。"
        End Try

        Return Nothing
    End Function

    ''' <summary>
    ''' ファイルが既に存在する場合に上書き確認を行います。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>ファイルが既に存在していて上書きしない場合はtrue、それ以外の場合はfalse。</returns>
    Private Shared Function ConfirmOverwrite(ByVal value As String) As Boolean
        If File.Exists(value) Then
            If MessageBox.Show(value & " は既に存在しています。" & Environment.NewLine &
                               "上書きしてよろしいですか？", "上書き確認",
                               MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                               MessageBoxDefaultButton.Button2) = DialogResult.No Then
                Return False
            End If
        End If

        Return True
    End Function

    ''' <summary>
    ''' 再検索可能までの残待機時間を更新します。
    ''' </summary>
    ''' <param name="waitingSeconds">秒単位の残り待機時間</param>
    Private Sub UpdateWaitingTime(ByVal waitingSeconds As Long)
        Dim labelText As String = String.Empty
        If waitingSeconds > 0 Then
            ' 待機時間が経過していない場合
            labelText = String.Format("再検索可能まで {0:00} 秒", waitingSeconds)
        End If
        ' コントロールの更新
        labelCountDown.Text = labelText
        buttonSearch.Enabled = waitingSeconds <= 0
        timerCalcWait.Enabled = waitingSeconds > 0
    End Sub

#End Region

    '==============================================================================================
    '  Event Handlers
    '==============================================================================================
#Region "Event Handlers"

    ''' <summary>
    ''' メインフォームのロードイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' 保存したユーザー設定を復元し設定する
        userSettings_ = UserSettings.Load(UserSettings.SettingFilePath)
        If userSettings_ Is Nothing Then
            ' 失敗した場合は既定値を設定
            userSettings_ = New UserSettings()
            textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl
            Return
        End If
        ApplyUserSettingsToControls(userSettings_)
    End Sub

    ''' <summary>
    ''' メインフォームのクローズ後イベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_FormClosed(sender As Object, e As FormClosedEventArgs) Handles MyBase.FormClosed
        ' ユーザー設定を保存する
        userSettings_ = GetUserSettingsFromControls()
        Try
            userSettings_.Save(UserSettings.SettingFilePath)
        Catch
            '
        End Try
    End Sub

    ''' <summary>
    ''' 「既定値」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonResetUrl_Click(sender As Object, e As EventArgs) Handles buttonResetUrl.Click
        textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl
    End Sub

    ''' <summary>
    ''' 「参照」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSelectInputFile_Click(sender As Object, e As EventArgs) Handles buttonSelectInputFile.Click
        textBoxInputFilePath.Text = SelectInputFile(textBoxInputFilePath.Text)
    End Sub

    ''' <summary>
    ''' 「参照」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSelectOutputFile_Click(sender As Object, e As EventArgs) Handles buttonSelectOutputFile.Click
        textBoxOutputFilePath.Text = SelectOutputFile(textBoxOutputFilePath.Text)
    End Sub

    ''' <summary>
    ''' 「検索」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSearch_Click(sender As Object, e As EventArgs) Handles buttonSearch.Click
        ' フォームから値を取得し検証
        userSettings_ = GetUserSettingsFromControls()
        Dim errorMessage As String = ValidateSettings(userSettings_.UserID, userSettings_.Password,
                                                      userSettings_.AdminUrl,
                                                      userSettings_.InputFilePath,
                                                      userSettings_.OutputFilePath)
        If errorMessage IsNot Nothing Then
            ' 設定にエラーが存在する場合
            textBoxResults.Text = errorMessage
            Return
        End If

        ' テキストボックスの表示を変更
        textBoxResults.Text = "検索中..."

        ' 「検索」ボタンを無効
        buttonSearch.Enabled = False

        ' AdminURLを確認してSSL使用フラグを変更
        CalcSettings.UseSsl = userSettings_.AdminUrl.StartsWith("https")

        ' ユーザーを作成
        user_ = New User(userSettings_.UserID, userSettings_.Password, userSettings_.AdminUrl,
                         CalcSettings.UseSsl)

        ' 検索の実行
        textBoxResults.Text = DoWork(userSettings_)

        ' 再検索までのカウントダウン
        timerCalcWait.Start()
    End Sub

    ''' <summary>
    ''' 「終了」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonExit_Click(sender As Object, e As EventArgs) Handles buttonExit.Click
        Close()
    End Sub

    ''' <summary>
    ''' 「マニュアルを開く」リンクのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub linkLabelManual_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles linkLabelManual.LinkClicked
        Try
            Process.Start(UserSettings.UsersGuideFilePath)
        Catch

        End Try
    End Sub

    ''' <summary>
    ''' 計算待機時間タイマーのイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub timerCalcWait_Tick(sender As Object, e As EventArgs) Handles timerCalcWait.Tick
        If user_.UserInfo Is Nothing Then
            UpdateWaitingTime(-1)
            Return
        End If
        Dim elpsedSeconds As Long = CLng((DateTime.Now - user_.LastGCTime).TotalSeconds)
        Dim waitingSeconds As Long = CLng(user_.UserInfo.GeoInterval) - elpsedSeconds
        UpdateWaitingTime(waitingSeconds)
    End Sub

#End Region


End Class
